% detemine the chromatins state based on target gene of histone modification data
function step1_1_calculate_enrichment_of_peak_overlap(peak_name_1, peak_name_2, workdir, overlap_exten, Nsamp, outputdir)
disp(['Simulating the enrichment of overlapping peaks of ' peak_name_1 ' from ' peak_name_2]);

clear peak peak1;
load([workdir, peak_name_1, '_bound_region.mat']);             % peak.
peak1 = peak;
npeak1 = length(peak1.chr_id);
[pvalue, Isort] = sort(peak1.pvalue, 'descend');

clear peak peak2;
load([workdir, peak_name_2, '_bound_region.mat']);
peak2 = peak;
npeak2 = length(peak2.chr_id);

marker_title = peak_name_1;
marker_title(marker_title=='_') = ' ';
    
target_title = peak_name_2;
target_title(target_title=='_') = ' ';

% generate random peaks
peakrand = cell(1, Nsamp);
for ns = 1:Nsamp
    peakrand{ns}.bpstart = peak1.bpstart;
end

nstep =100;
pstep = round(npeak1/nstep);
ovlp_freq = zeros(1,nstep);
ovlp_freq_perm_mean = zeros(1,nstep);
ovlp_freq_perm_std = zeros(1,nstep);
for pi = 1:nstep
    disp([num2str(pi) ' of ' num2str(nstep)]);
    clear peak;
    if pi==nstep
        pend = npeak1;
    else
        pend = pi*pstep;
    end
    peak.chr_id = peak1.chr_id(Isort(1:pend));
    peak.bpstart = peak1.bpstart(Isort(1:pend));
    peak.bpend = peak1.bpend(Isort(1:pend));
    
    % calculate real number of overlap peaks
    novlp = 0;
    chr_id_str = unique(peak.chr_id);
    ovlp_num_random = zeros(Nsamp, 1);
    for i = 1:length(chr_id_str)
        I0 = strcmp(peak1.chr_id, chr_id_str{i});
        region_start = min(peak1.bpstart(I0));
        region_len = max(peak1.bpstart(I0)) - region_start;
        
        I1 = find(strcmp(peak.chr_id, chr_id_str{i}));
        bpstart_t1 = peak.bpstart(I1) - overlap_exten;
        bpend_t1 = peak.bpend(I1) + overlap_exten;
        bplen_t1 = bpend_t1 - bpstart_t1 + 1;

        I2 = find(strcmp(peak2.chr_id, chr_id_str{i}));
        bpstart_t2 = peak2.bpstart(I2);
        bpend_t2 = peak2.bpend(I2);

        for m = 1:length(bpstart_t1)
            if ~isempty(find((bpend_t1(m)-bpstart_t2).*(bpend_t2-bpstart_t1(m))>=0, 1))  %overlap
                 novlp = novlp + 1;
            end
        end
        
        % generate random peaks
        nsel = length(I1);
        for ns = 1:Nsamp,
            bpstart_rand = region_start + floor(region_len * rand(nsel,1));
            bpend_rand = bpstart_rand + bplen_t1;
            for m = 1:length(bpstart_rand)
                if ~isempty(find((bpend_rand(m)-bpstart_t2).*(bpend_t2-bpstart_rand(m))>=0, 1))  
                    ovlp_num_random(ns) = ovlp_num_random(ns) + 1;
                end
            end
        end
    end
    
    ovlp_freq(pi)= novlp/pend;
    ovlp_freq_perm_mean(pi) = mean(ovlp_num_random./pend);
    ovlp_freq_perm_std(pi) = std(ovlp_num_random./pend);
end

figure(1)
clf;
plot([(1:nstep-1)*pstep, npeak1], ovlp_freq, 'r', 'linewidth', 2);nstep =100;
hold on;
errorbar([(1:nstep-1)*pstep, npeak1], ovlp_freq_perm_mean, ovlp_freq_perm_std, '--k');
xlabel('Peak ranked by intensity');
ylabel('Fraction of overlap');
legend(marker_title, 'random permuted peaks');
title(['overlap with ' target_title]);
saveas(gcf, [outputdir, peak_name_1, '_to_', peak_name_2, '_overlap_enrichment_plot.fig'], 'fig');
saveas(gcf, [outputdir, peak_name_1, '_to_', peak_name_2, '_overlap_enrichment_plot.tif'], 'tif');
